#
# spec file for package scalapack
#
# Copyright (c) 2025 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%global flavor openmpi4%{nil}

%define pname scalapack
%define so_ver  2
%global _lto_cflags %{_lto_cflags} -ffat-lto-objects

%if "%flavor" == ""
ExclusiveArch:  do_not_build
%define package_name %pname
%else
%define mpi_flavor %{flavor}
%endif

%if "%flavor" == "openmpi5"
ExcludeArch:    %{ix86} %{arm}
# Only build header back on one multibuild for non-HPC.
# Note: For HPC the headers need to be built always.
%{bcond_without blacs_devel_headers}
%else
%{bcond_with blacs_devel_headers}
%endif

%if 0%{!?package_name:1}
%define package_name %{pname}-%{mpi_flavor}
%endif
%define libname() lib%{pname}%{so_ver}-%{mpi_flavor}
%define libblacsname() libblacs%{so_ver}-%{mpi_flavor}
%define installdir %{_libdir}/mpi/gcc/%{mpi_flavor}
%define p_includedir %{_includedir}

Name:           %{package_name}
Version:        2.2.0
Release:        2.1
Summary:        A subset of LAPACK routines redesigned for heterogenous computing
# This is freely distributable without any restrictions.
License:        SUSE-Public-Domain
Group:          Development/Libraries/Parallel
URL:            http://www.netlib.org/scalapack/
Source0:        http://www.netlib.org/scalapack/%{pname}-%{version}.tgz
BuildRequires:  %{mpi_flavor}-devel
BuildRequires:  blas-devel
BuildRequires:  cmake >= 2.8
BuildRequires:  gcc-fortran
BuildRequires:  lapack-devel
%if %{without blacs_devel_headers}
BuildRequires:  blacs-devel-headers
%endif
BuildRoot:      %{_tmppath}/%{name}-%{version}-build

%description
The ScaLAPACK (or Scalable LAPACK) library includes a subset
of LAPACK routines redesigned for distributed memory MIMD
parallel computers. It is currently written in a
Single-Program-Multiple-Data style using explicit message
passing for interprocessor communication. It assumes
matrices are laid out in a two-dimensional block cyclic
decomposition.

ScaLAPACK is designed for heterogeneous computing and is
portable on any computer that supports MPI or PVM.

Like LAPACK, the ScaLAPACK routines are based on
block-partitioned algorithms in order to minimize the frequency
of data movement between different levels of the memory hierarchy.
(For such machines, the memory hierarchy includes the off-processor
memory of other processors, in addition to the hierarchy of registers,
cache, and local memory on each processor.) The fundamental building
blocks of the ScaLAPACK library are distributed memory versions (PBLAS)
of the Level 1, 2 and 3 BLAS, and a set of Basic Linear Algebra
Communication Subprograms (BLACS) for communication tasks that arise
frequently in parallel linear algebra computations. In the ScaLAPACK
routines, all interprocessor communication occurs within the PBLAS and the
BLACS. One of the design goals of ScaLAPACK was to have the ScaLAPACK
routines resemble their LAPACK equivalents as much as possible.

%package -n     blacs-devel-headers
Summary:        Development headers for BLACS
Group:          Development/Libraries/Parallel

%description -n blacs-devel-headers
This package contains headers for BLACS.

%package -n     %{libname %_vers}
Summary:        ScaLAPACK libraries compiled against %{mpi_flavor}%{?mpi_vers}
Group:          Development/Libraries/Parallel
Obsoletes:      %{name} < %{version}
Provides:       %{name} = %{version}

%description -n %{libname %_vers}
The ScaLAPACK (or Scalable LAPACK) library includes a subset
of LAPACK routines redesigned for distributed memory MIMD
parallel computers. It is currently written in a
Single-Program-Multiple-Data style using explicit message
passing for interprocessor communication. It assumes
matrices are laid out in a two-dimensional block cyclic
decomposition.

ScaLAPACK is designed for heterogeneous computing and is
portable on any computer that supports MPI or PVM.

Like LAPACK, the ScaLAPACK routines are based on
block-partitioned algorithms in order to minimize the frequency
of data movement between different levels of the memory hierarchy.
(For such machines, the memory hierarchy includes the off-processor
memory of other processors, in addition to the hierarchy of registers,
cache, and local memory on each processor.) The fundamental building
blocks of the ScaLAPACK library are distributed memory versions (PBLAS)
of the Level 1, 2 and 3 BLAS, and a set of Basic Linear Algebra
Communication Subprograms (BLACS) for communication tasks that arise
frequently in parallel linear algebra computations. In the ScaLAPACK
routines, all interprocessor communication occurs within the PBLAS and the
BLACS. One of the design goals of ScaLAPACK was to have the ScaLAPACK
routines resemble their LAPACK equivalents as much as possible.

This package contains ScaLAPACK	libraries compiled with	%{mpi_flavor}%{?mpi_vers}.

%package -n     %{libname %_vers}-devel
Summary:        Development libraries for ScaLAPACK (%{mpi_flavor}%{?mpi_vers})
Group:          Development/Libraries/Parallel
Requires:       %{libname %_vers} = %{version}
Requires:       %{mpi_flavor}-devel
%if "%{mpi_flavor}" == "openmpi5"
Provides:       lib%{pname}-openmpi-devel
%endif
Obsoletes:      %{name}-devel < %{version}
Provides:       %{name}-devel = %{version}

%description -n %{libname %_vers}-devel
This package contains development libraries for ScaLAPACK, compiled against %{mpi_flavor}%{?mpi_vers}.

%package -n     %{libname %_vers}-devel-static
Summary:        Static libraries for ScaLAPACK (%{mpi_flavor}%{?mpi_vers})
Group:          Development/Libraries/Parallel
Requires:       %{libname %_vers}-devel = %{version}
Obsoletes:      %{name}-devel-static < %{version}
Provides:       %{name}-devel-static = %{version}

%description -n %{libname %_vers}-devel-static
This package contains static libraries for ScaLAPACK, compiled against %{mpi_flavor}%{?mpi_vers}.

%package        test
Summary:        Test programs for ScaLAPACK (%{mpi_flavor}%{?mpi_vers})
Group:          Development/Libraries/Parallel

%description    test
This packages contains some test programs for ScaLAPACK compiled against
%{mpi_flavor}%{?mpi_vers}.

%package -n     %{libblacsname %_vers}
Summary:        Basic Linear Algebra Communication Subprograms
Group:          Development/Libraries/Parallel

%description -n %{libblacsname %_vers}
The BLACS (Basic Linear Algebra Communication Subprograms) project
provides a linear algebra oriented message passing interface for
a large range of distributed memory platforms.

The length of time required to implement efficient distributed memory
algorithms makes it impractical to rewrite programs for every new
parallel machine. The BLACS exist in order to make linear algebra
applications both easier to program and more portable.

%package -n     %{libblacsname %_vers}-devel
Summary:        Development libraries for BLACS (%{mpi_flavor}%{?mpi_vers})
Group:          Development/Libraries/Parallel
Requires:       %{libblacsname %_vers} = %{version}
Requires:       %{mpi_flavor}-devel
Requires:       blacs-devel-headers
Obsoletes:      blacs-%{mpi_flavor}-devel < %{version}
Provides:       blacs-%{mpi_flavor}-devel = %{version}

%description -n %{libblacsname %_vers}-devel
This package contains development libraries for BLACS, compiled against %{mpi_flavor}%{?mpi_vers}.

%package -n     %{libblacsname %_vers}-devel-static
Summary:        Development libraries for BLACS (%{mpi_flavor}%{?mpi_vers})
Group:          Development/Libraries/Parallel
Requires:       %{libblacsname %_vers}-devel = %{version}
%if "%{mpi_flavor}" == "openmpi5"
Provides:       libblacs%{so_ver}-openmpi-devel
%endif

%description -n %{libblacsname %_vers}-devel-static
This package contains static libraries for BLACS, compiled against %{mpi_flavor}%{?mpi_vers}.

%prep
%setup -q -n %{pname}-%{version}
cp SLmake.inc.example SLmake.inc
cat > %{_sourcedir}/baselibs.conf  <<EOF
%{libname %{?_vers}}
%{libname %{?_vers}}-devel
  requires -%{mpi_flavor}-<targettype>
  requires "%{libname %{?_vers}}-<targettype> = <version>"
EOF

%build
# implicit-function-declaration: https://github.com/Reference-ScaLAPACK/scalapack/issues/81
RPM_OPT_FLAGS="$RPM_OPT_FLAGS -Wno-error=implicit-function-declaration -fno-strict-aliasing -std=legacy"
echo $PATH | grep -q %{mpi_flavor} || \
    PATH=/usr/%_lib/mpi/gcc/%{mpi_flavor}/bin:$PATH
%define makeargs %{?_smp_flags}
#%%cmake -DCMAKE_C_FLAGS:STRING="$RPM_OPT_FLAGS -fPIC"
#    -DCMAKE_Fortran_FLAGS:STRING="$RPM_OPT_FLAGS -fPIC" \
#    -DBUILD_SHARED_LIBS:BOOL=ON -DBUILD_STATIC_LIBS:BOOL=ON \
#    -DCMAKE_INSTALL_LIBDIR:PATH=%%{installdir}/%%_lib
#%%make_jobs -C build
CC=mpicc
FC=mpif90
MYCFLAGS="$RPM_OPT_FLAGS -fPIC"
make lib CFLAGS="${MYCFLAGS}" CCFLAGS="${MYCFLAGS}" FCFLAGS="${MYCFLAGS}" %makeargs
cd TESTING/EIG;
make FCFLAGS="$RPM_OPT_FLAGS" %{makeargs}
cd ../LIN;
make FCFLAGS="$RPM_OPT_FLAGS" %{makeargs}

cd ../..
ar crs libblacs.a BLACS/SRC/*.o BLACS/SRC/*.oo
BLACS=blacs
for libname in scalapack ${BLACS}
do
${FC} -shared -Wl,--whole-archive lib${libname}.a  -Wl,--no-whole-archive \
       -Wl,-soname,lib${libname}.so.%{version} -o lib${libname}.so.%{version}
   ln -s lib${libname}.so.%{version} lib${libname}.so
   ln -s lib${libname}.so.%{version} lib${libname}.so.%{so_ver}
done

%install

#%%make_install -C build

mkdir -p %{buildroot}%{installdir}/%_lib
mkdir -p %{buildroot}%{installdir}/%{_lib}/TESTING

for f in *.a *.so*; do
    cp -f $f %{buildroot}%{installdir}/%_lib/$f
done
cp -f TESTING/x* TESTING/*.dat %{buildroot}%{installdir}/%{_lib}/TESTING

# blacs header
%if %{with blacs_devel_headers}
mkdir -p %{buildroot}%{p_includedir}/blacs/
install -m 644 BLACS/SRC/Bdef.h %{buildroot}%{p_includedir}/blacs/
%endif

# Copy docs
cp -f README LICENSE ../

%post -n %{libname %_vers} -p /sbin/ldconfig

%postun -n %{libname %_vers} -p/sbin/ldconfig

%post -n %{libblacsname %_vers} -p /sbin/ldconfig

%postun -n %{libblacsname %_vers} -p /sbin/ldconfig

%files -n %{libname %_vers}
%license LICENSE
%doc README
%{installdir}/%_lib/libscalapack.so.*

%files test
%{installdir}/%_lib/TESTING

%files -n %{libname %_vers}-devel
%{installdir}/%_lib/libscalapack.so

%files -n %{libname %_vers}-devel-static
%{installdir}/%_lib/libscalapack.a

%if %{with blacs_devel_headers}
%files -n blacs-devel-headers
%{p_includedir}/blacs/
%endif

%files -n %{libblacsname %_vers}
%{installdir}/%_lib/libblacs.so.*

%files -n %{libblacsname %_vers}-devel
%{installdir}/%_lib/libblacs.so

%files -n %{libblacsname %_vers}-devel-static
%{installdir}/%_lib/libblacs.a

%changelog
* Mon Mar 24 2025 Nicolas Morey <nicolas.morey@suse.com>
- Disable and remove support of gnu-hpc build flavours (bsc#1239982)
- openmpi5 is not available on 32b systems
- Switch default openmpi to openmpi4 to openmpi5
* Wed Sep 18 2024 Atri Bhattacharya <badshah400@gmail.com>
- Update to version 2.2.0:
  * Feature: Allow compilation in ILP64 mode
  * (gh#Reference-ScaLAPACK/scalapack#19).  Bugs fixed:
  - Use pointer macro to prevent integer overflow
    (gh#Reference-ScaLAPACK/scalapack#16).
  - Fix argument mismatches for implicitly defined functions
    (for gcc-10+ compatibility)
    (gh#Reference-ScaLAPACK/scalapack#26).
  - No memory leaks on blacs_exit
    (gh#Reference-ScaLAPACK/scalapack#37).
- Add `-Wno-error=implicit-function-declaration` to work around
  non-prototyped functions (gh#Reference-ScaLAPACK/scalapack#81).
* Fri Feb  2 2024 Atri Bhattacharya <badshah400@gmail.com>
- Enable openmpi5 and gnu-hpc-openmpi5 flavours.
* Mon Oct 23 2023 Egbert Eich <eich@suse.com>
- Disable HPC flavors on i586
* Mon Oct 23 2023 Nicolas Morey <nicolas.morey@suse.com>
- Drop support for obsolete openmpi[123]
- Prepare support for openmpi5
* Wed Feb  3 2021 Egbert Eich <eich@suse.com>
- Change 'Requires:' to other HPC packages to %%requires_eq to depend
  on the exact version. This should take care of HPC packages ignoring
  proper ABI versioning.
* Tue Jan  5 2021 Egbert Eich <eich@suse.com>
- Fix build for non-HPC packages: do not build BLACS headers for
  multiple flavors (boo#1178116).
* Thu Jul 23 2020 Egbert Eich <eich@suse.com>
- Add build support for gcc10 to HPC build (bsc#1174439).
* Thu Jul 16 2020 Egbert Eich <eich@suse.com>
- Add openmpi4 spport (provided by Alin M Elena)
- Replace -fallow-argument-mismatch by -std=legacy
* Wed Jun 17 2020 Egbert Eich <eich@suse.com>
- Add compiler flag -fallow-argument-mismatch to allow build with
  gcc10.
* Sat Mar 28 2020 Egbert Eich <eich@suse.com>
- Remove blacs HPC master packages: the libblacs has been removed
  (boo#1167949).
* Thu Dec  5 2019 Egbert Eich <eich@suse.com>
- Add a Provides: for the removed BLACS library package for HPC.
  libscalapack contains the BLACS ABI.
* Sat Nov 30 2019 Egbert Eich <eich@suse.com>
- Updated to v2.1.0 (jsc#SLE-11037)
  * New robust ScaLAPACK routine for computing the QR factorization with
    column pivoting by Zvonimir Bujanovic and Zlatko Drmac [Download Lawn 296]
  * MPI-3+ updates required by Open MPI 4
  * Improve accuracy of Frobenius norm by making intermediate column sums
  * miscellaneous minor bug fixes
  * Improve cmake shared library support
- Disable openmpi1 builds for SLE/Leap > 15.1.
- Enable openmpi3 builds for Leap and SLE > 15.1 (jsc#SLE-7773).
- Add support for gcc8 and gcc9 builds for HPC (jsc#SLE-7766,
  jsc#SLE-8604).
- For HPC no longer provide BLACS as a separate library.
- Remove:
  * scalapack-2.0.2-shared-blacs.patch
  * scalapack-2.0.2-shared-lib.patch
    Handled in spec file.
* Mon Nov 25 2019 Stefan Brüns <stefan.bruens@rwth-aachen.de>
- Reflect renaming of openmpi packages to openmpi1 in Tumbleweed,
  fixes unresolvable packages.
* Tue Sep 10 2019 Ana Guerrero Lopez <aguerrero@suse.com>
- Use -ffat-lto-objects to provide a proper static library.
* Tue Jul 16 2019 Michel Normand <normand@linux.vnet.ibm.com>
- Add BuildRequires blacs-devel-headers for non-hpc when required.
- Add BuildRequires libopenblas-gnu-hpc >=  0.3.6 for hpc to  avoid
  to build when openblas library not available.
* Fri Feb 22 2019 Stefan Brüns <stefan.bruens@rwth-aachen.de>
- spec file cleanup:
  * Use %%license, remove %%defattr
  * Shorten descriptions and fix some typos
  * Remove redundant (and wrong) pointer to test data install dir
* Wed Feb 13 2019 Egbert Eich <eich@suse.com>
- Consolidate use of openmpi1, openmpi2, openmpi3 for non-HPC builds.
- Disable openmpi1/3 and non-HPC builds on SLE.
* Wed Dec 12 2018 Todd R <toddrme2178@gmail.com>
- Also build for regular openmpi2 and openmpi3.
* Fri May 11 2018 eich@suse.com
- Fix HPC build that broke with last change: the BLACS headers
  need to be provided always (boo#1092881).
* Thu Apr 26 2018 jberry@suse.com
- Only produce blacs-devel-headers package once (in openmpi)
  (boo#1092881).
* Sat Feb 10 2018 eich@suse.com
- Fix setting of
  * LIBRARY_PATH
  * PKG_CONFIG_PATH (boo#1080426).
  * Fix headers (boo#1080434).
* Fri Feb  9 2018 eich@suse.com
- Since the OpenBLAS build was fixed, reenable scalapack
  on s390 (bsc#1079513).
- Add support for openmpi2 for HPC (FATE#325089).
* Fri Feb  9 2018 cgoll@suse.com
- Fix summary in module files (bnc#1080259)
* Tue Feb  6 2018 eich@suse.com
- The HPC build of scalapack requires openBLAS. OpenBLAS is not
  supported for s390: skip building on s390 for HPC (bsc#1079513).
* Wed Jan 17 2018 eich@suse.com
- Don't set the module package to noarch. It contains arch specific
  directory paths (boo#1076443).
* Fri Jan 12 2018 eich@suse.com
- Disable the openmpi3 flavor in some products.
* Fri Jan  5 2018 eich@suse.com
- Switch from gcc6 to gcc7 as additional compiler flavor for HPC on SLES.
- Add support for openmpi3 and mpich to HPC build.
* Tue Nov 14 2017 cgoll@suse.com
- changed location of test files in order to make tests functional
* Fri Nov 10 2017 eich@suse.com
- Add libopenblas*devel as Requires to the devel package for
  HPC.
* Thu Oct 19 2017 eich@suse.com
- Fix blacs master packages.
* Mon Oct 16 2017 eich@suse.com
- Make sure the Factory validator gets %%pname as package name.
* Fri Sep  8 2017 eich@suse.com
- Convert to multibuild
- Add HPC build using environment modules
  (FATE#321715).
- Generate baselib.conf dynamically and only for the non-HPC
  builds: this avoids issues with the source validator.
* Fri Jul 24 2015 dmitry_r@opensuse.org
- Fix BLACS devel package dependence list
* Tue Jul  7 2015 dmitry_r@opensuse.org
- Build BLACS shared libraries
  * scalapack-2.0.2-shared-blacs.patch
* Wed Nov  5 2014 dmitry_r@opensuse.org
- Add baselibs.conf to source list
* Wed Jun  4 2014 dev@stellardeath.org
- Fix shared library, the *.oo files were missing
* Fri Aug  3 2012 scorot@free.fr
- fix requires/obsoletes
* Sun Jun  3 2012 scorot@free.fr
- add baselibs.conf
* Sat May 19 2012 scorot@free.fr
- delete common subpackage
- add lib suffix to packages containing libs
- build test programs and them in test subpackage
- minor fixes in the Requirements of some devel packages
* Sun May 13 2012 scorot@free.fr
- remove Provides and Obsoletes flags
* Sun May 13 2012 scorot@free.fr
- version 2.0.2
  * miscellaneous minor bug fixes
  * bug fixes to the PxHSEQR Nonsymmetric Eigenvalue Problem
    routines
  * new xLAMOV routines replaces calls to xLACPY to avoid array
    argument overlap
  * modifications to BLACS to better support MPICH-based MPI
    libraries
  * new PMPIM2 and PMPCOL routines to avoid code duplication in MRRR
    routines
- package provides blacs since blacs is included in scalapack
  since version 2.0.0
- remove mpich2 stuff
* Fri May  4 2012 scorot@free.fr
- spec file cleanup
- remove unneeded blas and lapack from buildReq
- remove SUSE_ASNEEDED environement flag
